@@ -5,12 +5,14 @@
 #
 #__all__=['devsuit', 'android', 'image', 'base', 'patch', 'ios', 'device']
 
-__version__ = '0.9.0'
+__version__ = '0.9.4'
 
 ANDROID = 'android'
 IOS = 'ios'
 WINDOWS='windows'
 
+ANDROIDWIFI = 'androidwifi'
+
 EV_DOWN = 'down'
 EV_UP = 'up'
 EV_DOWN_AND_UP = 'down_and_up'
@@ -110,6 +112,9 @@ def connect(devno=None, appname=None, device=None, monitor=True, logfile='log/ai
     elif device == 'dummy': # this class is only for test
         from airtest.device import dummy
         devClass = dummy.Device 
+    elif device == ANDROIDWIFI:
+        from airtest.device import androidwifi
+        devClass = androidwifi.Device
     else:
         raise RuntimeError('device type not recognize')
 
