@@ -5,14 +5,15 @@ import collections
 import os
 import platform
 import time
+import threading
 import json
 from PIL import Image
 
-from airtest import base
-from airtest import jsonlog
-
-from airtest import image as imt
-# from airtest.image import auto as image
+from . import base
+from . import proto
+from .image import auto as imtauto
+from .image import sift as imtsift
+from .image import template as imttemplate
 
 import airtest
 
@@ -34,16 +35,6 @@ def rotate_point((x, y), (w, h), d):
     if d == 'LEFT':
         return h-y, x
 
-def get_jsonlog(filename='log/airtest.log'):    
-    logfile = os.getenv('AIRTEST_LOGFILE', filename)
-    if os.path.exists(logfile):
-        backfile = logfile+'.'+time.strftime('%Y%m%d%H%M%S')
-        os.rename(logfile, backfile)
-    else:
-        base.makedirs(base.dirname(logfile))
-    jlog = jsonlog.JSONLog(logfile)
-    return jlog
-
 class DeviceSuit(object):
     def __init__(self, device, devClass, phoneno, 
             appname=None, logfile='log/airtest.log', monitor=True):
@@ -63,16 +54,22 @@ class DeviceSuit(object):
         self._threshold = 0.3 # for findImage
 
         self._rotation = None # UP,DOWN,LEFT,RIGHT
-        self._log = get_jsonlog(logfile).writeline # should implementes writeline(dict)
         self._tmpdir = 'tmp'
-        self._log(dict(type='start', timestamp=time.time()))
-        self._configfile = os.getenv('AIRTEST_CONFIG') or 'air.json'
         self._click_timeout = 20.0 # if icon not found in this time, then panic
         self._delay_after_click = 0.5 # when finished click, wait time
         self._screen_resolution = None
 
         self._snapshot_file = None
         self._keep_capture = False # for func:keepScreen,releaseScreen
+        self._logfile = logfile
+        self._loglock = threading.Lock()
+
+        logdir = os.path.dirname(logfile) or '.'
+        if not os.path.exists(logdir):
+            os.makedirs(logdir)
+        if os.path.exists(logfile):
+            backfile = logfile+'.'+time.strftime('%Y%m%d%H%M%S')
+            os.rename(logfile, backfile)
 
         #-- start of func setting
         self._init_monitor()
@@ -90,12 +87,17 @@ class DeviceSuit(object):
         #-- end of func setting
 
     def __getattribute__(self, name):
+        # print name
         v = object.__getattribute__(self, name)
         if isinstance(v, collections.Callable):
             objdict = object.__getattribute__(self, '__dict__')
+            # print objdict
             def _wrapper(*args, **kwargs):
                 objdict['_inside_depth'] += 1
+                # log function call
                 ret = v(*args, **kwargs)
+                if objdict['_inside_depth'] == 1 and not v.__name__.startswith('_'):
+                    self.log(proto.TAG_FUNCTION, dict(name=v.__name__, args=args, kwargs=kwargs))
                 objdict['_inside_depth'] -= 1
                 return ret
             return _wrapper
@@ -105,11 +107,10 @@ class DeviceSuit(object):
         def _cpu_mem_monitor():
             if not self.appname:
                 return
-            mem = self.dev.getMem(self.appname)
-            self._log({'type':'record', 'mem':mem.get('PSS', 0)/1024})
-            self._log({'type':'record', 'mem_details':mem})
-            cpu = self.dev.getCpu(self.appname)
-            self._log({'type':'record', 'cpu':cpu})
+            meminfo = self.dev.meminfo(self.appname)
+            self.log(proto.TAG_MEMORY, meminfo)
+            cpuinfo = self.dev.cpuinfo(self.appname)
+            self.log(proto.TAG_CPU, cpuinfo)
 
         self.monitor = airtest.monitor.Monitor()
         self.monitor.addfunc(_cpu_mem_monitor)
@@ -118,11 +119,11 @@ class DeviceSuit(object):
     def _imfind(self, bgimg, search):
         method = self._image_match_method
         if method == 'auto':
-            point = imt.auto.locate_one_image(bgimg, search, threshold=self._threshold)
+            point = imtauto.locate_one_image(bgimg, search, threshold=self._threshold)
         elif method == 'template':
-            point = imt.template.find(search, bgimg, self._threshold)
+            point = imttemplate.find(search, bgimg, self._threshold)
         elif method == 'sift':
-            point = imt.sift.find(search, bgimg)
+            point = imtsift.find(search, bgimg)
         else:
             raise RuntimeError("Unknown image match method: %s" %(method))
         print 'find method=', method
@@ -133,11 +134,11 @@ class DeviceSuit(object):
             maxcnt = 0
         method = self._image_match_method
         if method == 'auto':
-            points = imt.auto.locate_more_image_Template(search, bgimg, num=maxcnt)
+            points = imtauto.locate_more_image_Template(search, bgimg, num=maxcnt)
         elif method == 'template':
-            points = imt.template.findall(search, bgimg, self._threshold, maxcnt=maxcnt)
+            points = imttemplate.findall(search, bgimg, self._threshold, maxcnt=maxcnt)
         elif method == 'sift':
-            points = imt.sift.findall(search, bgimg, maxcnt=maxcnt)
+            points = imtsift.findall(search, bgimg, maxcnt=maxcnt)
         else:
             raise RuntimeError("Unknown image match method: %s" %(method))
         if sort:
@@ -233,10 +234,25 @@ class DeviceSuit(object):
         if rotation != 'UP':
             angle = dict(RIGHT=Image.ROTATE_90, LEFT=Image.ROTATE_270).get(rotation)
             Image.open(filename).transpose(angle).save(filename)
-        self._log(dict(type='snapshot', filename=filename))
+        if tempdir:
+            self.log(proto.TAG_SNAPSHOT, dict(filename=filename))
         self._snapshot_file = filename
         return filename
 
+    def log(self, tag, message):
+        self._loglock.acquire()
+        timestamp = time.time()
+        try:
+            dirname = os.path.dirname(self._logfile) or '.'
+            if not os.path.exists(dirname):
+                os.path.makedirs(dirname)
+        except:
+            pass
+        with open(self._logfile, 'a') as file:
+            data = dict(timestamp=int(timestamp), tag=tag, data=message)
+            file.write(json.dumps(data) + '\n')
+        self._loglock.release()
+
     def keepCapture(self):
         '''
         Use screen in memory
@@ -257,7 +273,6 @@ class DeviceSuit(object):
         @return string: (filename that really save to)
         '''
         savefile = self._saveScreen(filename, random_name=False, tempdir=False)
-        self._log(dict(type='snapshot', filename=savefile))
         return savefile
 
     def globalSet(self, *args, **kwargs):
@@ -368,31 +383,23 @@ class DeviceSuit(object):
     def exists(self, imgfile):
         return True if self.find(imgfile) else False
 
-    def click(self, SF, seconds=None, eventType=airtest.EV_DOWN_AND_UP):
+    def click(self, SF, seconds=None, duration=0.1):
         '''
         Click function
         @param seconds: float (if time not exceed, it will retry and retry)
         '''
         if seconds == None:
             seconds = self._click_timeout
-        log.info('CLICK %s, timeout=%.2f', SF, seconds)
+        log.info('CLICK %s, timeout=%.2fs, duration=%.2fs', SF, seconds, duration)
         point = self._PS2Point(SF)
         if point:
             (x, y) = point
-            self.dev.touch(x, y, eventType)
         else:
             (x, y) = self.wait(SF, seconds=seconds)
             log.info('Click %s point: (%d, %d)', SF, x, y)
-            self.dev.touch(x, y, eventType)
+        self.dev.touch(x, y, duration)
         log.debug('delay after click: %.2fs' ,self._delay_after_click)
 
-        # mark position
-        # import cv2
-        # img = cv2.imread(self._snapshot_file)
-        # if img != None:
-        #     img = imt.toolbox.markPoint(img, (x, y))
-        #     cv2.imwrite(self._snapshot_file, img)
-
         time.sleep(self._delay_after_click)
 
     def center(self):
@@ -418,31 +425,15 @@ class DeviceSuit(object):
             log.debug('ignore for no exists %s', imgfile)
             return False
 
-    def drag(self, fpt, tpt, duration=500):
+    def drag(self, fpt, tpt, duration=0.5):
         ''' 
         Drag from one place to another place
 
         @param fpt,tpt: filename or position
-        @param duration: float (duration of the event in ms)
+        @param duration: float (duration of the event in seconds)
         '''
-        # the duration seems not working. no matter how larger I set, nothing changes.
-        # variable = {}
-        # def to_point(raw):
-        #     if isinstance(raw, list) or isinstance(raw, tuple):
-        #         return self._fixPoint(raw)
-        #     if isinstance(raw, basestring):
-        #         screen = variable.get('screen')
-        #         if not screen:
-        #             variable['screen'] = self._saveScreen('drag-XXXXXXXX.png')
-        #         pt = find_one_image(variable['screen'], raw, self._threshold)
-        #         return self._fixPoint(pt)
-        #     raise RuntimeError('unknown type')
-        # FIXME: (a little slow, find should support specified images)
         fpt = self._PS2Point(fpt)
         tpt = self._PS2Point(tpt)
-        # fpt = self.find(fpt)
-        # tpt = self.find(tpt)
-        # fpt, tpt = to_point(fpt), to_point(tpt)
         return self.dev.drag(fpt, tpt, duration)
 
     def sleep(self, secs=1.0):
@@ -482,39 +473,4 @@ class DeviceSuit(object):
             return self.dev.keyevent(event)
         raise RuntimeError('keyevent not support')
 
-    def close(self):
-        '''
-        Release resouces
-        '''
-        self.monitor.stop()
-        time.sleep(0.5)
-
-    def _safe_load_config(self):
-        import os
-        if os.path.exists(self._configfile):
-            return json.load(open(self._configfile))
-        return {}
-            
-    def start(self):
-        '''
-        Start a app
-        '''
-        s = self._safe_load_config()
-        ret = self.dev.start(self.appname, s.get(self._device))
-        self._initWidthHeight()
-        return ret
-    
-    def stop(self):
-        '''
-        Stop a app
-        '''
-        s = self._safe_load_config()
-        return self.dev.stop(self.appname, s.get(self._device))
-
-    def clear(self):
-        '''
-        Stop app and clear data
-        '''
-        s = self._safe_load_config()
-        return self.dev.clear(self.appname, s.get(self._device))
 
